# -*- encoding: utf-8 -*-
##############################################################################
#
#    Asterisk 360 module for OpenERP
#    Copyright (C) 2012 Les Green <l.green@valuedecision.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from osv import osv, fields
from tools.translate import _


class res_partner_address(osv.osv):
    _inherit = "res.partner.address"
    
    def setup_call_inbound(self, cr, uid, number, extension, context=None):
        details = self.get_phone_details(cr, uid, number, context)
        details['extension'] = extension
        details['openerp_call_id'] = self.create_inbound_call(cr, uid, details, number)
        return details
 
    def get_phone_details(self, cr, uid, number, context=None):
        return self.get_phone_details_base(cr, uid, number, context)

    def _get_search_from_number(self, number):
        ret = number.lstrip('0')
        if len(ret) >= 9: return ret[-9:len(ret)] #match last 9 digits only 
        return ret  

    def get_phone_details_base (self, cr, uid, number, context=None):
        test_num = self._get_search_from_number(number)
        sql = """
select 
  rpa.id as address_id, 
  rpa.name as name, 
  rpa.partner_id as partner_id,
  rp.name as partner_name, 
  regexp_replace(phone,'[^0-9]','','g') as phone, 
  regexp_replace(mobile,'[^0-9]','','g') as mobile 
from res_partner_address rpa 
left outer join res_partner rp
on rpa.partner_id = rp.id
where regexp_replace(phone,'[^0-9]','','g') like '%%%s%%' or regexp_replace(mobile,'[^0-9]','','g') like '%%%s%%'
""" % (test_num, test_num)
        cr.execute(sql)
        res = cr.fetchall()
        pst = {}
        for item in res:
            if item[0]: pst['address_id'] = item[0]
            if item[1]: pst['address_name'] = item[1]
            if item[2]: pst['partner_id'] = item[2]
            if item[3]: pst['partner_name'] = item[3]
            
        pst['type'] = 'partner' if 'partner_id' in pst else 'address' 
        return pst
  
    def create_inbound_call(self, cr, uid, details, inbound_number):
        inbound_category_id = self.pool.get('crm.case.categ').search(cr, uid, [('name', '=', 'Inbound')])
        return self.create_call(cr, uid, details, inbound_number, inbound_category_id)

    def _get_caller_name(self, details):
        
        person = details['address_name'] if 'address_name' in details else False
        person = details['contact_name'] if 'contact_name' in details else person
        partner = details['partner_name'] if 'partner_name' in details else False
        
        if person:
            if partner: return '%s (%s)' % (person, partner)
            return person
        
        if partner: return partner
        
        return 'Unknown'
        
    def create_call(self, cr, uid, details, number, category_id):

        vals = {
         'name': '%s %s' % (self._get_caller_name(details),_('Inbound')),
         'partner_phone': number,
         'categ_id': category_id[0],
        }
        if 'extension' in details: 
            user_obj = self.pool.get('res.users')
            search_results = user_obj.search(cr, uid, [('internal_number','=',details['extension'])])
            if len(search_results)>0:
                vals['user_id'] = search_results[0]
                section_id = user_obj.read(cr, uid, vals['user_id'],['context_section_id'])['context_section_id']
                if section_id:
                    vals['section_id'] = section_id[0]
            
        if 'partner_id' in details: vals['partner_id'] = details['partner_id']
        if 'address_id' in details: vals['partner_address_id'] = details['address_id']
              
        call_object = self.pool.get('crm.phonecall')
        call_id = call_object.create(cr, uid, vals)
        call_object.case_open(cr, uid, [call_id])
        return call_id
res_partner_address()

class asterisk_server(osv.osv):
    _inherit = "asterisk.server"

    _columns = {
        'ws_address': fields.char('Asterisk WebSocket Server Full Address or DNS', size=150, help="Full URI of the WebSockets Server including port  e.g ws://<ip>:<port>")
    }
asterisk_server()

# Parameters specific for each user
class res_users(osv.osv):
    _inherit = "res.users"
    _columns = {
        'asterisk_chan_type': fields.selection([('SIP', 'SIP'), ('IAX2', 'IAX2'), ('DAHDI', 'DAHDI'), ('Zap', 'Zap'), ('Skinny', 'Skinny'), ('MGCP', 'MGCP'), ('mISDN', 'mISDN'), ('H323', 'H323'), ('USTM', 'USTM'), ('SCCP', 'SCCP')], 'Asterisk channel type', help="Asterisk channel type, as used in the Asterisk dialplan. If the user has a regular IP phone, the channel type is 'SIP'."),
	'screen_pop': fields.boolean('Screen Pop', help="Check to enable screen pops for this user")
    }
res_users()
