# -*- coding: utf-8 -*-
##############################################################################
#
#    Asterisk 360 module for OpenERP
#    Copyright (C) 2012 Les Green <l.green@valuedecision.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from osv import fields, osv
from tools.translate import _
import logging
import socket
import time

_logger = logging.getLogger(__name__)

class asterisk_server(osv.osv):
    _inherit = 'asterisk.server'
    
    def _get_outbound_summary(self,details):
        
        summary = 'Outbound'
        person = details['name'] if 'name' in details else False
        partner = details['partner_id'][1] if 'partner_id' in details else False
        
        if person:
            if partner: return _('%s (%s) outbound') % (person, partner)
            return _('%s outbound') % person
        elif partner:
            return _('%s outbound') % partner
        
        return summary
        

    def dial(self, cr, uid, ids, erp_number, details, contact_id, context=None):
        '''
        Open the socket to the Asterisk Manager Interface (AMI)
        and send instructions to Dial to Asterisk. That's the important function !
        '''
        _logger.debug('Intercepted Asterisk Call')
            
        # Log an outbound call
        outboundCategoryId = self.pool.get('crm.case.categ').search(cr, uid,[('name','=','Outbound')])
        call_vals = {
                     'name': self._get_outbound_summary(details),
                     'partner_phone': erp_number,
                     'categ_id': outboundCategoryId[0],
                         }
        if contact_id:
            call_vals['partner_address_id'] = contact_id
            
            if 'partner_id' in details:
                call_vals['partner_id'] = details['partner_id'][0]
         
            user_obj = self.pool.get('res.users')
            section_id = user_obj.read(cr, uid, uid,['context_section_id'])['context_section_id']
            if section_id:
                call_vals['section_id'] = section_id[0]
            
            openerp_call_id = self.pool.get('crm.phonecall').create(cr, uid, call_vals)
        else:
            if not 'id' in details:
                raise osv.except_osv(_('Error :'), _('No call id passed to dial.'))

            openerp_call_id = details['id']
            
            updated = self.pool.get('crm.phonecall').write(cr, uid,openerp_call_id,{'date':time.strftime('%Y-%m-%d %H:%M:%S')})
            if not updated:
                raise osv.except_osv(_('Error :'), _('Failed to update the start time of the scheduled phonecall.'))

        
        _logger.debug('Got Call id %s' % openerp_call_id)
            
        context['no_dial_button']=True

        options = { 'erp_number': erp_number, 'openerp_call_id': openerp_call_id }
        self._connect_to_asterisk(cr, uid, method='dial', options=options, context=context)
              
        return {
                'name':_("Dialled Call"),
                'view_mode': 'form',
                'view_type': 'tree,form',
                'res_model': 'crm.phonecall',
                'res_id': openerp_call_id,
                'views': [(False,'form')],
                'type': 'ir.actions.act_window',
                'nodestroy': True,
                'target': 'current',
                'context': context
        }
    
    #Override this only really to send the call id we are using - consider adding feature to branch of click2dial
    def _connect_to_asterisk(self, cr, uid, method='dial', options=None, context=None):
        '''
        Open the socket to the Asterisk Manager Interface (AMI)
        and send instructions to Dial to Asterisk. That's the important function !

        '''
        user = self.pool.get('res.users').browse(cr, uid, uid, context=context)

        # Note : if I write 'Error' without ' :', it won't get translated...
        # I don't understand why !

        ast_server = self._get_asterisk_server_from_user(cr, uid, user, context=context)
        # We check if the current user has a chan type
        if not user.asterisk_chan_type:
            raise osv.except_osv(_('Error :'), _('No channel type configured for the current user.'))

        # We check if the current user has an internal number
        if not user.internal_number:
            raise osv.except_osv(_('Error :'), _('No internal phone number configured for the current user'))


        _logger.debug("User's phone : %s/%s" % (user.asterisk_chan_type, user.internal_number))
        _logger.debug("Asterisk server = %s:%d" % (ast_server.ip_address, ast_server.port))

        # Connect to the Asterisk Manager Interface, using IPv6-ready code
        try:
            res = socket.getaddrinfo(str(ast_server.ip_address), ast_server.port, socket.AF_UNSPEC, socket.SOCK_STREAM)
        except:
            _logger.warning("Can't resolve the DNS of the Asterisk server '%s'" % ast_server.ip_address)
            raise osv.except_osv(_('Error :'), _("Can't resolve the DNS of the Asterisk server : '%s'" % ast_server.ip_address))
        for result in res:
            af, socktype, proto, canonname, sockaddr = result
            sock = socket.socket(af, socktype, proto)
            try:
                sock.connect(sockaddr)
                header_received = sock.recv(1024)
                _logger.debug('Header received from Asterisk : %s' % header_received)

                # Login to Asterisk
                login_act = 'Action: login\r\n' + \
                    'Events: off\r\n' + \
                    'Username: ' + ast_server.login + '\r\n' + \
                    'Secret: ' + ast_server.password + '\r\n\r\n'
                sock.send(login_act.encode('ascii'))
                login_answer = self._parse_asterisk_answer(cr, uid, sock, context=context)
                if 'Response: Success' in login_answer:
                    _logger.debug("Successful authentification to Asterisk :\n%s" % login_answer)
                else:
                    raise osv.except_osv(_('Error :'), _("Authentification to Asterisk failed :\n%s" % login_answer))

                if method == 'dial':
                    # Convert the phone number in the format that will be sent to Asterisk
                    erp_number = options.get('erp_number')
                    if not erp_number:
                        raise osv.except_osv(_('Error :'), "Hara kiri : you must call the function with erp_number in the options")
                    ast_number = self._reformat_number(cr, uid, erp_number, ast_server, context=context)

                    # The user should have a CallerID
                    if not user.callerid:
                        raise osv.except_osv(_('Error :'), _('No callerID configured for the current user'))

                    # Dial with Asterisk
                    originate_act = 'Action: originate\r\n' + \
                        'Channel: ' + user.asterisk_chan_type + '/' + user.internal_number + ( ('/' + user.dial_suffix) if user.dial_suffix else '') + '\r\n' + \
                        'Priority: ' + str(ast_server.extension_priority) + '\r\n' + \
                        'Timeout: ' + str(ast_server.wait_time*1000) + '\r\n' + \
                        'CallerId: ' + user.callerid + '\r\n' + \
                        'Exten: ' + ast_number + '\r\n' + \
                        'Context: ' + ast_server.context + '\r\n'
                    if ast_server.alert_info and user.asterisk_chan_type == 'SIP':
                        for server_alertinfo in ast_server.alert_info.split('|'):
                            originate_act += 'Variable: SIPAddHeader=Alert-Info: ' + server_alertinfo.strip() + '\r\n'
                    if user.alert_info and user.asterisk_chan_type == 'SIP':
                        for user_alertinfo in user.alert_info.split('|'):
                            originate_act += 'Variable: SIPAddHeader=Alert-Info: ' + user_alertinfo.strip() + '\r\n'
                    if user.variable:
                        for user_variable in user.variable.split('|'):
                            originate_act += 'Variable: ' + user_variable.strip() + '\r\n'
                    #Extended here
                    call_id_info = 'Variable: OPENERP_CALL_ID=%s\r\n' % options.get('openerp_call_id')
                    originate_act += call_id_info
                    #end
                    
                    originate_act += '\r\n'
                    sock.send(originate_act.encode('ascii'))
                    originate_answer = self._parse_asterisk_answer(cr, uid, sock, context=context)
                    if 'Response: Success' in originate_answer:
                        _logger.debug('Successfull originate command : %s' % originate_answer)
                    else:
                        raise osv.except_osv(_('Error :'), _("Click to dial with Asterisk failed :\n%s" % originate_answer))

                else:
                    raise osv.except_osv(_('Error :'), _("Click to dial extension only supports the dial method"))

                # Logout of Asterisk
                sock.send(('Action: Logoff\r\n\r\n').encode('ascii'))
                logout_answer = self._parse_asterisk_answer(cr, uid, sock, context=context)
                if 'Response: Goodbye' in logout_answer:
                    _logger.debug('Successfull logout from Asterisk :\n%s' % logout_answer)
                else:
                    _logger.warning('Logout from Asterisk failed :\n%s' % logout_answer)
            # we catch only network problems here
            except socket.error:
                _logger.warning("Unable to connect to the Asterisk server '%s' IP '%s:%d'" % (ast_server.name, ast_server.ip_address, ast_server.port))
                raise osv.except_osv(_('Error :'), _("The connection from OpenERP to the Asterisk server failed. Please check the configuration on OpenERP and on Asterisk."))
            finally:
                sock.close()
        if method == 'dial':
            _logger.info("Asterisk Click2Dial from %s/%s to %s" % (user.asterisk_chan_type, user.internal_number, ast_number))
            return True
        else:
            return False
asterisk_server()

class res_partner_address(osv.osv):
    _inherit = 'res.partner.address'
    
    def action_dial_phone(self, cr, uid, ids, context=None):
        '''Function called by the button 'Dial' next to the 'phone' field
        in the partner address view'''
        details = self.read(cr, uid, ids, ['phone','partner_id','name'], context=context)[0]
        return self.pool.get('asterisk.server').dial(cr, uid, ids, details['phone'], details, ids[0], context=context)
        
    def action_dial_mobile(self, cr, uid, ids, context=None):
        '''Function called by the button 'Dial' next to the 'mobile' field
        in the partner address view'''
        details = self.read(cr, uid, ids, ['mobile','partner_id','name'], context=context)[0]
        return self.pool.get('asterisk.server').dial(cr, uid, ids, details['mobile'], details, ids[0], context=context)

res_partner_address()

class crm_phonecall(osv.osv):
    _inherit = 'crm.phonecall'
    
    def action_dial_phone(self, cr, uid, ids, context=None):
        '''Function called by the button 'Dial' in the Scheduled Call field 
        in the phonecall'''
        details = self.read(cr, uid, ids, ['partner_id','partner_phone'], context=context)[0]
        return self.pool.get('asterisk.server').dial(cr, uid, ids, details['partner_phone'], details, False, context=context)

    def action_dial_mobile(self, cr, uid, ids, context=None):
        '''Function called by the button 'Dial' in the Scheduled Call field 
        in the phonecall'''
        details = self.read(cr, uid, ids, ['partner_id','partner_mobile'], context=context)[0]
        return self.pool.get('asterisk.server').dial(cr, uid, ids, details['partner_mobile'], details, False, context=context)
    
crm_phonecall()